/*****************************************************
  COMPLEX FUNCTIONS AND OTHER SPECIAL FUNCTIONS

COPYRIGHT:
     Copyright 1994 by David Boas (boas@sol1.lrsm.upenn.edu)

     Permission to use, copy, modify, and distribute this
     software and its documentation for any purpose and without
     fee is hereby granted, provided that the above copyright
     notice appear in all copies.  The University of Pennsylvania
     makes no representations about the suitability of this
     software for any purpose.  It is provided "as is" without
     express or implied warranty.

DISCLAIMER:
     UNDER NO CIRCUMSTANCES WILL THE AUTHOR OF THIS SOFTWARE OR THE
     UNIVERSITY OF PENNSYLVANIA BE RESPONSIBLE FOR ANY DIRECT OR
     INCIDENTAL DAMAGE ARISING FROM THE USE OF THIS SOFTWARE AND ITS
     DOCUMENTATION. THE SOFTWARE HEREIN IS PROVIDED "AS IS" WITH NO
     IMPLIED OBLIGATION TO PROVIDE SUPPORT, UPDATES, OR MODIFICATIONS.
*****************************************************/

#include "p1.h"


/*****************************************************
  SPHERICAL HARMONICS
*****************************************************/
dcomplex SphHarmonic( l, theta )
int l;
double theta;
{
  double p, p1, p2, x;
  int i;

  x = cos( theta );
  p2 = 1;
  p1 = x;

  if( l==0 ) return Complex(sqrt(1/(4*3.14159))*p2,0.);
  if( l==1 ) return Complex(sqrt(3/(4*3.14159))*p1,0.);
  for( i=1; i<l; i++ )
    {
      p = ((2*i+1)*x*p1 - i*p2)/(i+1);
      p2 = p1;
      p1 = p;
    }
  return Complex(sqrt((2*l+1)/(4*3.14159))*p,0.);
}

/*****************************************************
  COMPLEX EXPONENTIAL
*****************************************************/
dcomplex Cexp(x)
dcomplex x;
{
  dcomplex z;
  z.r = cos(x.i) * exp(x.r);
  z.i = sin(x.i) * exp(x.r);
  return z;
}

/*****************************************************
  COMPLEX COSINE
*****************************************************/
dcomplex Ccos(x)
dcomplex x;
{
  dcomplex z;
  z.r = ( cos(x.r) * exp(-x.i) + cos(x.r) * exp(x.i) ) / 2.0;
  z.i = ( sin(x.r) * exp(-x.i) - sin(x.r) * exp(x.i) ) / 2.0;
  return z;
}

/*****************************************************
  COMPLEX SINE
*****************************************************/
dcomplex Csin(x)
dcomplex x;
{
  dcomplex z;
  z.r = ( sin(x.r) * exp(-x.i) + sin(x.r) * exp(x.i) ) / 2.0;
  z.i = -1 * ( cos(x.r) * exp(-x.i) - cos(x.r) * exp(x.i) ) / 2.0;
  return z;
}

/*****************************************************
  COMPLEX LOG
*****************************************************/
dcomplex Clog(x)
dcomplex x;
{
  dcomplex z;
  z = Complex( log(Cabs(x)), atan2(x.i,x.r) );
  return z;
}


