/*
  quad.c

  Numerical evalution of integrals (quadrature).
  This function approximates the integral of f(x) from a to b
  to within a relative error of tol.  quad uses an adaptive
  recursive Simpson's rule.

  This function has been adapted from Matlab.
*/
#include "p1.h"

#define FUNC(x) ((*func)(x))
#define LEVMAX 30

double q;    /* This makes the routine specific for an integrand
		which decays fast enough */


dcomplex quad( func, a, b, tol )
double a, b, tol;
dcomplex (*func)();
{
  int lev;
  double c;
  dcomplex Q0, Q, fa,fb,fc;
  dcomplex quadstp();

  c = (a+b)/2.;
  fa = FUNC(a);
  fb = FUNC(b);
  fc = FUNC(c);
  
  lev = 1;
  Q0 = Complex(1.e100,1.e100);
  q = 0.;
  Q = quadstp( func, a, b, tol, lev, fa, fc, fb, Q0 );

  return Q;
}

dcomplex quadstp( func, a, b, tol, lev, fa, fc, fb, Q0 )
int lev;
double a, b, tol;
dcomplex fa, fb, fc, Q0;
dcomplex (*func)();
{
  double foo1, foo2;
  double h,c;
  dcomplex f1,f2,Q1,Q2,Q;

  if( lev>LEVMAX ) {
    printf( "Recursion limit reached in quadstp.  Singularity likely.\n" );
    exit(1);
  }

  h = b-a;
  c = (a+b)/2.;
  f1 = FUNC(a+h/4.);
  f2 = FUNC(b-h/4.);

/*Simpson's rule for half intervals*/
  Q1 = RCmul(h/12.,Cadd(fa,Cadd(RCmul(4.,f1),fc))); 
  Q2 = RCmul(h/12.,Cadd(fc,Cadd(RCmul(4.,f2),fb))); 
  Q = Cadd( Q1, Q2 );
  if( q==0. )  q=Cabs(Q);

  foo1 = Cabs(Csub(Q,Q0));
  foo2 = Cabs(Q);

/*recursively refine approximations*/
  if( foo1 > tol*foo2 && foo2/q > tollerence ) {
    Q1 = quadstp( func, a, c, tol, lev+1, fa, f1, fc, Q1 );
    Q2 = quadstp( func, c, b, tol, lev+1, fc, f2, fb, Q2 );
    Q = Cadd( Q1, Q2 );
  }

  return Q;
}
